#include "AccelStepper.h"

// ****************************************************************************************
// **                                                                                    **
// **  Stepper Servo for Synthiam ARC (c)2022 Synthian Inc.                              **
// **  October 5, 2022                                                                   **
// **                                                                                    **
// **  Things you have to do:                                                            **
// **  - Read the manual for the robot skill and configure this code                     **
// **  - https://synthiam.com/Support/Skills/Servo/Stepper-Servo?id=21134                **
// **                                                                                    **
// ****************************************************************************************


// **************************************************************************************************************************************
// The ID (1 - 127) of this stepper controller on the serial bus. There can be up to 127 of these stepper servo controllers on a single UART bus
// If two or more these have the same ID, they will both respond to the same commands.
// **************************************************************************************************************************************
#define BUS_ID 1


// **************************************************************************************************************************************
// Configure the steps per second on initialization. This is when the arduino is first powered on.
// This value can be modified in ARC by using the Speed parameter as well.
// **************************************************************************************************************************************
#define SPEED 100


// **************************************************************************************************************************************
// Configure the acceleration on initialization. This is when the arduino is first powered on.
// This value can be modified in ARC by using the Acceleration parameter as well.
// **************************************************************************************************************************************
#define ACCELERATION 100


// **************************************************************************************************************************************
// Calibrate position using endstop limit switch on power-up
// This will begin moving the stepper until the specified endstop limit switch is reached, then it will set that to ARC servo position 1
// Set this to false if you do not have a limit switch, or you wish to run this feature programatically by controlcommand in ARC
// Options are 1 to calibrate on power on, or 0 to not calibrate.
// **************************************************************************************************************************************
#define CALIBRATE_ON_POWER 0


// **************************************************************************************************************************************
// Direction to move when calibrating toward the endstop limit switch.
// The options are 0 for decrement (counter clockwise) and 1 for increment (clockwise).
// If you're not using a limit switch, it does not matter what value is here.
// **************************************************************************************************************************************
#define CALIBRATION_DIRECTION 1


// **************************************************************************************************************************************
// Number of steps to move when calibrating
// **************************************************************************************************************************************
#define CALIBRATION_STEPS 50


// **************************************************************************************************************************************
// Optional endstop limit switch port
// This switch is expected to be shorted to GND when the switch is closed. This pin will be held high internally until grounded
// If not using a limit switch, it does not matter what value is here.
// **************************************************************************************************************************************
#define END_STOP_LIMIT_PORT 6


// **************************************************************************************************************************************
// Configure the stepper driver type below
// Only one of these option sections below can be uncommented. If you are going to change from one driver type to another, make sure
// you comment the old section and uncomment the new section.
// **************************************************************************************************************************************

// Uncomment the four lines below if using stepper DRIVER mode (with Step and Direction pins).
//#define STEPPER_TYPE DRIVER
//#define STEPPER1_STEP_PIN 2
//#define STEPPER1_DIR_PIN 3
//AccelStepper stepper1(AccelStepper::DRIVER, STEPPER1_STEP_PIN, STEPPER1_DIR_PIN);


// Uncomment the four lines below if using a stepper driver in FULL2WIRE mode a 2 wire stepper (2 pins required).
//#define STEPPER_TYPE FULLWIRE
//#define STEPPER1_WIRE1 2
//#define STEPPER1_WIRE2 3
//AccelStepper stepper1(AccelStepper::FULL2WIRE, STEPPER1_WIRE1, STEPPER1_WIRE2);


// Uncomment the five lines below if using a stepper driver in FULL3WIRE mode a 3 wire stepper, such as HDD spindle (3 pins required).
//#define STEPPER_TYPE FULL3WIRE
//#define STEPPER1_WIRE1 2
//#define STEPPER1_WIRE2 3
//#define STEPPER1_WIRE3 4
//AccelStepper stepper1(AccelStepper::FULL3WIRE, STEPPER1_WIRE1, STEPPER1_WIRE2, STEPPER1_WIRE3);


// Uncomment the six lines below if using a stepper driver in FULL4WIRE means a 4 wire stepper (4 pins required).
//#define STEPPER_TYPE FULL4WIRE
//#define STEPPER1_WIRE1 2
//#define STEPPER1_WIRE2 5
//#define STEPPER1_WIRE3 3
//#define STEPPER1_WIRE4 4
//AccelStepper stepper1(AccelStepper::FULL4WIRE, STEPPER1_WIRE1, STEPPER1_WIRE2, STEPPER1_WIRE3, STEPPER1_WIRE4);


// Uncomment the five lines below if using a stepper driver in HALF3WIRE means a 3 wire half stepper, such as HDD spindle (3 pins required).
//#define STEPPER_TYPE HALF3WIRE
//#define STEPPER1_WIRE1 2
//#define STEPPER1_WIRE2 3
//#define STEPPER1_WIRE3 4
//AccelStepper stepper1(AccelStepper::HALF3WIRE, STEPPER1_WIRE1, STEPPER1_WIRE2, STEPPER1_WIRE3);


// Uncomment the six lines below if using a stepper driver in HALF4WIRE means a 4 wire half stepper (4 pins required).
//#define STEPPER_TYPE HALF4WIRE
//#define STEPPER1_WIRE1 2
//#define STEPPER1_WIRE2 4
//#define STEPPER1_WIRE3 3
//#define STEPPER1_WIRE4 5
//AccelStepper stepper1(AccelStepper::HALF4WIRE, STEPPER1_WIRE1, STEPPER1_WIRE2, STEPPER1_WIRE3, STEPPER1_WIRE4);


// ********************************************************************************************************************************
// You may also invert the pins. Uncomment this to invert the control lines.
// This is if the driver expects a LOW instead of HIGH for trigger, they can be inverted.
// Warning, you can damage the stepper motor or driver if this is set and not needed.
// ********************************************************************************************************************************
//#define INVERT


// ********************************************************************************************************************************
// No edit below here
// ********************************************************************************************************************************

#ifndef STEPPER_TYPE
#error "No stepper configuration was set. Check the manual for this robot skill on synthiam.com"
#endif

void setup() {

  Serial.begin(115200);

  stepper1.setMaxSpeed(SPEED);
  stepper1.setAcceleration(ACCELERATION);

#ifdef INVERT
  stepper1.setPinsInverted();
#endif

  pinMode(END_STOP_LIMIT_PORT, INPUT_PULLUP);

#if (CALIBRATE_ON_POWER == 1)
  calibrate();
#endif
}

void calibrate() {

  while (digitalRead(END_STOP_LIMIT_PORT)) {

    long pos = stepper1.currentPosition();

    if (stepper1.currentPosition() == stepper1.targetPosition()) {

#if (CALIBRATION_DIRECTION == 0)
      stepper1.move(-CALIBRATION_STEPS);
#elif (CALIBRATION_DIRECTION == 1)
      stepper1.move(CALIBRATION_STEPS);
#endif
    }

    stepper1.runSpeed();
  }

  stepper1.setCurrentPosition(0);
}

byte readBlocking() {

  int val = Serial.read();

  while (val == -1)
    val = Serial.read();

  return val;
}

void write32(long val) {

  Serial.write((byte)((val >> 24) & 0xff));
  Serial.write((byte)((val >> 16) & 0xff));
  Serial.write((byte)((val >> 8) & 0xff));
  Serial.write((byte)(val & 0xff));
}

void loop() {

  while (Serial.available()) {

    if (readBlocking() != 0x55)
      break;

    if (readBlocking() != 0xAA)
      break;

    if (readBlocking() != BUS_ID)
      break;

    byte cmd = readBlocking();

    if (cmd == 0) {

      // Set as new home position
      stepper1.setCurrentPosition(0);

    } else if (cmd == 1) {

      // Set max speed

      int s = readBlocking();

      s *= 2;

      stepper1.setMaxSpeed(s);

    } else if (cmd == 2) {

      // Set acceleration

      byte s = readBlocking();

      stepper1.setAcceleration(s);

    } else if (cmd == 3) {

      // Set position

      long pos = (readBlocking() << 24) | (readBlocking() << 16) | (readBlocking() << 8) | readBlocking();

      stepper1.moveTo(pos);

    } else if (cmd == 4) {

      // Stop moving panic!
      stepper1.stop();

    } else if (cmd == 5) {

      // calibrate endlimit switch
      calibrate();

    } else if (cmd == 6) {

      // query position
      Serial.write(0x55);
      Serial.write(0xAA);
      write32(stepper1.currentPosition());
      
    }
  }

  stepper1.run();
}
